/*
 * Copyright (C) 2020 Purism SPC
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 * Author: Guido Günther <agx@sigxcpu.org>
 */

#pragma once

#include <glib.h>

G_BEGIN_DECLS

void             phosh_log_set_log_domains (const char *domains);

G_END_DECLS
